@@ -193,14 +193,17 @@ individual files from the archive.\
 
                 for (l = files; l != NULL; l = l->next) {
                     if (list_details) {
+                        g_autofree char *date = NULL;
                         g_autoptr(GDateTime) dt = gcab_file_get_date_time (GCAB_FILE (l->data));
-                        g_autoptr(GDateTime) dtl = g_date_time_to_local (dt);
-                        g_autofree char *date = g_date_time_format (dtl, "%Y-%m-%d %H:%M:%S");
+                        if (dt) {
+                            g_autoptr(GDateTime) dtl = g_date_time_to_local (dt);
+                            date = g_date_time_format (dtl, "%Y-%m-%d %H:%M:%S");
+                        }
 
                         g_print ("%s %u %s 0x%X\n",
                                  gcab_file_get_name (GCAB_FILE (l->data)),
                                  gcab_file_get_size (GCAB_FILE (l->data)),
-                                 date,
+                                 date ? date : _("<unknown-date>"),
                                  gcab_file_get_attributes (GCAB_FILE (l->data)));
                     } else {
                         g_print ("%s\n", gcab_file_get_name (GCAB_FILE (l->data)));
